@@ -42,27 +42,6 @@ lxc_log_define(lxc_caps, lxc);
 #define PR_CAPBSET_READ 23
 #endif
 
-/* Control the ambient capability set */
-#ifndef PR_CAP_AMBIENT
-#define PR_CAP_AMBIENT 47
-#endif
-
-#ifndef PR_CAP_AMBIENT_IS_SET
-#define PR_CAP_AMBIENT_IS_SET 1
-#endif
-
-#ifndef PR_CAP_AMBIENT_RAISE
-#define PR_CAP_AMBIENT_RAISE 2
-#endif
-
-#ifndef PR_CAP_AMBIENT_LOWER
-#define PR_CAP_AMBIENT_LOWER 3
-#endif
-
-#ifndef PR_CAP_AMBIENT_CLEAR_ALL
-#define PR_CAP_AMBIENT_CLEAR_ALL 4
-#endif
-
 int lxc_caps_down(void)
 {
 	cap_t caps;
@@ -114,6 +93,7 @@ int lxc_caps_up(void)
 	}
 
 	for (cap = 0; cap <= CAP_LAST_CAP; cap++) {
+
 		cap_flag_value_t flag;
 
 		ret = cap_get_flag(caps, cap, CAP_PERMITTED, &flag);
@@ -146,115 +126,6 @@ out:
 	return 0;
 }
 
-int lxc_ambient_caps_up(void)
-{
-	int ret;
-	cap_t caps;
-	cap_value_t cap;
-	int last_cap = CAP_LAST_CAP;
-	char *cap_names = NULL;
-
-	if (!getuid() || geteuid())
-		return 0;
-
-	caps = cap_get_proc();
-	if (!caps) {
-		SYSERROR("Failed to retrieve capabilities");
-		return -1;
-	}
-
-	for (cap = 0; cap <= CAP_LAST_CAP; cap++) {
-		cap_flag_value_t flag;
-
-		ret = cap_get_flag(caps, cap, CAP_PERMITTED, &flag);
-		if (ret < 0) {
-			if (errno == EINVAL) {
-				last_cap = (cap - 1);
-				INFO("Last supported cap was %d", last_cap);
-				break;
-			}
-
-			SYSERROR("Failed to retrieve capability flag");
-			goto out;
-		}
-
-		ret = cap_set_flag(caps, CAP_INHERITABLE, 1, &cap, flag);
-		if (ret < 0) {
-			SYSERROR("Failed to set capability flag");
-			goto out;
-		}
-	}
-
-	ret = cap_set_proc(caps);
-	if (ret < 0) {
-		SYSERROR("Failed to set capabilities");
-		goto out;
-	}
-
-	for (cap = 0; cap <= last_cap; cap++) {
-		ret = prctl(PR_CAP_AMBIENT, PR_CAP_AMBIENT_RAISE, cap, 0, 0);
-		if (ret < 0) {
-			WARN("%s - Failed to raise ambient capability %d",
-			     strerror(errno), cap);
-			goto out;
-		}
-	}
-
-	cap_names = cap_to_text(caps, NULL);
-	if (!cap_names) {
-		SYSWARN("Failed to convert capabilities %d", cap);
-		goto out;
-	}
-
-	TRACE("Raised %s in inheritable and ambient capability set", cap_names);
-
-out:
-
-	cap_free(cap_names);
-	cap_free(caps);
-	return 0;
-}
-
-int lxc_ambient_caps_down(void)
-{
-	int ret;
-	cap_t caps;
-	cap_value_t cap;
-
-	if (!getuid() || geteuid())
-		return 0;
-
-	ret = prctl(PR_CAP_AMBIENT, PR_CAP_AMBIENT_CLEAR_ALL, 0, 0, 0);
-	if (ret < 0) {
-		SYSERROR("Failed to clear ambient capability set");
-		return -1;
-	}
-
-	caps = cap_get_proc();
-	if (!caps) {
-		SYSERROR("Failed to retrieve capabilities");
-		return -1;
-	}
-
-	for (cap = 0; cap <= CAP_LAST_CAP; cap++) {
-		ret = cap_set_flag(caps, CAP_INHERITABLE, 1, &cap, CAP_CLEAR);
-		if (ret < 0) {
-			SYSERROR("Failed to remove capability from inheritable set");
-			goto out;
-		}
-	}
-
-	ret = cap_set_proc(caps);
-	if (ret < 0) {
-		SYSERROR("Failed to set capabilities");
-		goto out;
-	}
-
-out:
-	cap_free(caps);
-	return 0;
-}
-
 int lxc_caps_init(void)
 {
 	uid_t uid = getuid();
@@ -313,14 +184,9 @@ static int _real_caps_last_cap(void)
 		char *ptr;
 		int n;
 
-	again:
-		n = read(fd, buf, 31);
-		if (n < 0 && errno == EINTR) {
-			goto again;
-		} else if (n >= 0) {
+		if ((n = read(fd, buf, 31)) >= 0) {
 			buf[n] = '\0';
 			errno = 0;
-
 			result = strtol(buf, &ptr, 10);
 			if (!ptr || (*ptr != '\0' && *ptr != '\n') || errno != 0)
 				result = -1;
@@ -333,10 +199,7 @@ static int _real_caps_last_cap(void)
 	* each capability indiviually from the kernel */
 	if (result < 0) {
 		int cap = 0;
-
-		while (prctl(PR_CAPBSET_READ, cap) >= 0)
-			cap++;
-
+		while (prctl(PR_CAPBSET_READ, cap) >= 0) cap++;
 		result = cap - 1;
 	}
 
@@ -346,9 +209,7 @@ static int _real_caps_last_cap(void)
 int lxc_caps_last_cap(void)
 {
 	static int last_cap = -1;
-
-	if (last_cap < 0)
-		last_cap = _real_caps_last_cap();
+	if (last_cap < 0) last_cap = _real_caps_last_cap();
 
 	return last_cap;
 }
