@@ -17,23 +17,20 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 1
-#endif
+#define _GNU_SOURCE
 #define __STDC_FORMAT_MACROS /* Required for PRIu64 to work. */
 #include <errno.h>
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <sys/socket.h>
 #include <sys/un.h>
-#include <unistd.h>
 
 #include "af_unix.h"
 #include "commands.h"
 #include "commands_utils.h"
-#include "config.h"
 #include "initutils.h"
 #include "log.h"
 #include "lxclock.h"
@@ -55,7 +52,7 @@ int lxc_cmd_sock_rcv_state(int state_client_fd, int timeout)
 		ret = setsockopt(state_client_fd, SOL_SOCKET, SO_RCVTIMEO,
 				(const void *)&out, sizeof(out));
 		if (ret < 0) {
-			SYSERROR("Failed to set %ds timeout on container "
+			SYSERROR("Failed to set %ds timeout on containter "
 				 "state socket",
 				 timeout);
 			return -1;
@@ -76,6 +73,9 @@ again:
 		return -1;
 	}
 
+	if (ret < 0)
+		return -1;
+
 	TRACE("Received state %s from state client %d",
 	      lxc_state2str(msg.value), state_client_fd);
 
@@ -101,38 +101,24 @@ int lxc_cmd_sock_get_state(const char *name, const char *lxcpath,
 	return ret;
 }
 
-int lxc_make_abstract_socket_name(char *path, size_t pathlen,
-				  const char *lxcname,
+int lxc_make_abstract_socket_name(char *path, int len, const char *lxcname,
 				  const char *lxcpath,
 				  const char *hashed_sock_name,
 				  const char *suffix)
 {
 	const char *name;
-	char *offset;
 	char *tmppath;
-	size_t len;
 	size_t tmplen;
 	uint64_t hash;
 	int ret;
 
-	if (!path)
-		return -1;
-
-	offset = &path[1];
-
-	/* -2 here because this is an abstract unix socket so it needs a
-	 * leading \0, and we null terminate, so it needs a trailing \0.
-	 * Although null termination isn't required by the API, we do it anyway
-	 * because we print the sockname out sometimes.
-	 */
-	len = pathlen - 2;
-
 	name = lxcname;
 	if (!name)
 		name = "";
 
 	if (hashed_sock_name != NULL) {
-		ret = snprintf(offset, len, "lxc/%s/%s", hashed_sock_name, suffix);
+		ret =
+		    snprintf(path, len, "lxc/%s/%s", hashed_sock_name, suffix);
 		if (ret < 0 || ret >= len) {
 			ERROR("Failed to create abstract socket name");
 			return -1;
@@ -148,7 +134,7 @@ int lxc_make_abstract_socket_name(char *path, size_t pathlen,
 		}
 	}
 
-	ret = snprintf(offset, len, "%s/%s/%s", lxcpath, name, suffix);
+	ret = snprintf(path, len, "%s/%s/%s", lxcpath, name, suffix);
 	if (ret < 0) {
 		ERROR("Failed to create abstract socket name");
 		return -1;
@@ -166,7 +152,7 @@ int lxc_make_abstract_socket_name(char *path, size_t pathlen,
 	}
 
 	hash = fnv_64a_buf(tmppath, ret, FNV1A_64_INIT);
-	ret = snprintf(offset, len, "lxc/%016" PRIx64 "/%s", hash, suffix);
+	ret = snprintf(path, len, "lxc/%016" PRIx64 "/%s", hash, suffix);
 	if (ret < 0 || ret >= len) {
 		ERROR("Failed to create abstract socket name");
 		return -1;
@@ -179,17 +165,27 @@ int lxc_cmd_connect(const char *name, const char *lxcpath,
 		    const char *hashed_sock_name, const char *suffix)
 {
 	int ret, client_fd;
-	char path[LXC_AUDS_ADDR_LEN] = {0};
+	char path[sizeof(((struct sockaddr_un *)0)->sun_path)] = {0};
+	char *offset = &path[1];
 
-	ret = lxc_make_abstract_socket_name(path, sizeof(path), name, lxcpath,
+	/* -2 here because this is an abstract unix socket so it needs a
+	 * leading \0, and we null terminate, so it needs a trailing \0.
+	 * Although null termination isn't required by the API, we do it anyway
+	 * because we print the sockname out sometimes.
+	 */
+	size_t len = sizeof(path) - 2;
+	ret = lxc_make_abstract_socket_name(offset, len, name, lxcpath,
 					    hashed_sock_name, suffix);
 	if (ret < 0)
 		return -1;
 
 	/* Get new client fd. */
 	client_fd = lxc_abstract_unix_connect(path);
-	if (client_fd < 0)
+	if (client_fd < 0) {
+		if (errno == ECONNREFUSED)
+			return -ECONNREFUSED;
 		return -1;
+	}
 
 	return client_fd;
 }
@@ -198,7 +194,7 @@ int lxc_add_state_client(int state_client_fd, struct lxc_handler *handler,
 			 lxc_state_t states[MAX_STATE])
 {
 	int state;
-	struct state_client *newclient;
+	struct lxc_state_client *newclient;
 	struct lxc_list *tmplist;
 
 	newclient = malloc(sizeof(*newclient));
@@ -218,13 +214,13 @@ int lxc_add_state_client(int state_client_fd, struct lxc_handler *handler,
 	state = handler->state;
 	if (states[state] != 1) {
 		lxc_list_add_elem(tmplist, newclient);
-		lxc_list_add_tail(&handler->state_clients, tmplist);
+		lxc_list_add_tail(&handler->conf->state_clients, tmplist);
 	} else {
 		free(newclient);
 		free(tmplist);
 		return state;
 	}
 
-	TRACE("Added state client %d to state client list", state_client_fd);
+	TRACE("added state client %d to state client list", state_client_fd);
 	return MAX_STATE;
 }
