@@ -36,7 +36,9 @@
 #include <stdbool.h>
 
 #include "list.h"
+#include "ringbuf.h"
 #include "start.h" /* for lxc_handler */
+#include "terminal.h"
 
 #if HAVE_SCMP_FILTER_CTX
 typedef void * scmp_filter_ctx;
@@ -51,6 +53,8 @@ typedef void * scmp_filter_ctx;
  * programmer to specify the right subsystem.
  * @subsystem : the targeted subsystem
  * @value     : the value to set
+ * @version   : The version of the cgroup filesystem on which the controller
+ *              resides.
  *
  * @controllers : The controllers to use for this container.
  * @dir         : The name of the directory containing the container's cgroup.
@@ -60,6 +64,7 @@ struct lxc_cgroup {
 	union {
 		/* information about a specific controller */
 		struct /* controller */ {
+			int version;
 			char *subsystem;
 			char *value;
 		};
@@ -94,6 +99,26 @@ enum idtype {
 	ID_TYPE_GID
 };
 
+/*
+ * Defines a structure to configure kernel parameters at runtime.
+ * @key      : the kernel parameters will be configured without the "lxc.sysctl" prefix
+ * @value    : the value to set
+ */
+struct lxc_sysctl {
+	char *key;
+	char *value;
+};
+
+/*
+ * Defines a structure to configure proc filesystem at runtime.
+ * @filename : the proc filesystem will be configured without the "lxc.proc" prefix
+ * @value    : the value to set
+ */
+struct lxc_proc {
+	char *filename;
+	char *value;
+};
+
 /*
  * id_map is an id map entry.  Form in confile is:
  * lxc.idmap = u 0    9800 100
@@ -109,53 +134,16 @@ struct id_map {
 	unsigned long hostid, nsid, range;
 };
 
-/*
- * Defines a structure containing a pty information for
- * virtualizing a tty
- * @name   : the path name of the slave pty side
- * @master : the file descriptor of the master
- * @slave  : the file descriptor of the slave
- */
-struct lxc_pty_info {
-	char name[MAXPATHLEN];
-	int master;
-	int slave;
-	int busy;
-};
-
-/*
- * Defines the number of tty configured and contains the
+/* Defines the number of tty configured and contains the
  * instantiated ptys
  * @nbtty = number of configured ttys
  */
 struct lxc_tty_info {
 	int nbtty;
-	struct lxc_pty_info *pty_info;
+	struct lxc_terminal_info *tty;
 };
 
-struct lxc_tty_state;
-
-/*
- * Defines the structure to store the console information
- * @peer   : the file descriptor put/get console traffic
- * @name   : the file name of the slave pty
- */
-struct lxc_console {
-	int slave;
-	int master;
-	int peer;
-	struct lxc_pty_info peerpty;
-	struct lxc_epoll_descr *descr;
-	char *path;
-	char *log_path;
-	int log_fd;
-	char name[MAXPATHLEN];
-	struct termios *tios;
-	struct lxc_tty_state *tty_state;
-};
-
-/*
- * Defines a structure to store the rootfs location, the
+/* Defines a structure to store the rootfs location, the
  * optionals pivot_root, rootfs mount paths
  * @path       : the rootfs source (directory or device)
  * @mount      : where it is mounted
@@ -195,9 +183,9 @@ enum {
 	 * variants, which is safe. */
 	LXC_AUTO_CGROUP_NOSPEC        = 0x0B0,   /* /sys/fs/cgroup (partial mount, r/w or mixed, depending on caps) */
 	LXC_AUTO_CGROUP_FULL_NOSPEC   = 0x0E0,   /* /sys/fs/cgroup (full mount, r/w or mixed, depending on caps) */
-	LXC_AUTO_CGROUP_MASK          = 0x0F0,
-
-	LXC_AUTO_ALL_MASK             = 0x0FF,   /* all known settings */
+	LXC_AUTO_CGROUP_FORCE         = 0x100,   /* mount cgroups even when cgroup namespaces are supported */
+	LXC_AUTO_CGROUP_MASK          = 0x1F0,   /* all known cgroup options, doe not contain LXC_AUTO_CGROUP_FORCE */
+	LXC_AUTO_ALL_MASK             = 0x1FF,   /* all known settings */
 };
 
 /*
@@ -212,7 +200,7 @@ enum {
  * @fstab      : path to a fstab file format
  * @caps       : list of the capabilities to drop
  * @keepcaps   : list of the capabilities to keep
- * @tty_info   : tty data
+ * @ttys       : tty data
  * @console    : console data
  * @ttydir     : directory (under /dev) in which to create console and ttys
  * @lsm_aa_profile : apparmor profile to switch to or NULL
@@ -228,11 +216,17 @@ enum lxchooks {
 	LXCHOOK_POSTSTOP,
 	LXCHOOK_CLONE,
 	LXCHOOK_DESTROY,
+	LXCHOOK_START_HOST,
 	NUM_LXC_HOOKS
 };
 
 extern char *lxchook_names[NUM_LXC_HOOKS];
 
+struct lxc_state_client {
+	int clientfd;
+	lxc_state_t states[MAX_STATE];
+};
+
 struct lxc_conf {
 	int is_execute;
 	char *fstab;
@@ -241,21 +235,38 @@ struct lxc_conf {
 	int reboot;
 	signed long personality;
 	struct utsname *utsname;
-	struct lxc_list cgroup;
-	struct lxc_list id_map;
+	struct {
+		struct lxc_list cgroup;
+		struct lxc_list cgroup2;
+	};
+	struct {
+		struct lxc_list id_map;
+
+		/* Pointer to the idmap entry for the container's root uid in
+		 * the id_map list. Do not free! */
+		struct id_map *root_nsuid_map;
+
+		/* Pointer to the idmap entry for the container's root gid in
+		 * the id_map list. Do not free! */
+		struct id_map *root_nsgid_map;
+	};
 	struct lxc_list network;
 	int auto_mounts;
 	struct lxc_list mount_list;
 	struct lxc_list caps;
 	struct lxc_list keepcaps;
-	struct lxc_tty_info tty_info;
+	struct lxc_tty_info ttys;
 	/* Comma-separated list of lxc.tty.max pty names. */
 	char *pty_names;
-	struct lxc_console console;
+	struct lxc_terminal console;
 	struct lxc_rootfs rootfs;
 	char *ttydir;
 	int close_all_fds;
-	struct lxc_list hooks[NUM_LXC_HOOKS];
+
+	struct {
+		unsigned int hooks_version;
+		struct lxc_list hooks[NUM_LXC_HOOKS];
+	};
 
 	char *lsm_aa_profile;
 	unsigned int lsm_aa_allow_incomplete;
@@ -281,8 +292,6 @@ struct lxc_conf {
 	int loglevel; /* loglevel as specifed in config (if any) */
 	int logfd;
 
-	int inherit_ns_fd[LXC_NS_MAX];
-
 	unsigned int start_auto;
 	unsigned int start_delay;
 	int start_order;
@@ -308,6 +317,9 @@ struct lxc_conf {
 	char *unexpanded_config;
 	size_t unexpanded_len, unexpanded_alloced;
 
+	/* default command for lxc-execute */
+	char *execute_cmd;
+
 	/* init command */
 	char *init_cmd;
 
@@ -329,20 +341,35 @@ struct lxc_conf {
 	/* RLIMIT_* limits */
 	struct lxc_list limits;
 
-	/* REMOVE IN LXC 3.0
-	 * Indicator whether the current config file we're using contained any
-	 * legacy configuration keys.
-	 */
-	bool contains_legacy_key;
-
 	/* Contains generic info about the cgroup configuration for this
 	 * container. Note that struct lxc_cgroup contains a union. It is only
 	 * valid to access the members of the anonymous "meta" struct within
 	 * that union.
 	 */
 	struct lxc_cgroup cgroup_meta;
+
+	struct {
+		int ns_clone;
+		int ns_keep;
+		char *ns_share[LXC_NS_MAX];
+	};
+
+	/* init working directory */
+	char *init_cwd;
+
+	/* A list of clients registered to be informed about a container state. */
+	struct lxc_list state_clients;
+
+	/* sysctls */
+	struct lxc_list sysctls;
+
+	/* procs */
+	struct lxc_list procs;
 };
 
+extern int write_id_mapping(enum idtype idtype, pid_t pid, const char *buf,
+			    size_t buf_size);
+
 #ifdef HAVE_TLS
 extern __thread struct lxc_conf *current_config;
 #else
@@ -350,17 +377,17 @@ extern struct lxc_conf *current_config;
 #endif
 
 extern int run_lxc_hooks(const char *name, char *hook, struct lxc_conf *conf,
-			 const char *lxcpath, char *argv[]);
+			 char *argv[]);
 extern int detect_shared_rootfs(void);
 extern struct lxc_conf *lxc_conf_init(void);
 extern void lxc_conf_free(struct lxc_conf *conf);
 extern int pin_rootfs(const char *rootfs);
 extern int lxc_map_ids(struct lxc_list *idmap, pid_t pid);
 extern int lxc_create_tty(const char *name, struct lxc_conf *conf);
-extern void lxc_delete_tty(struct lxc_tty_info *tty_info);
+extern void lxc_delete_tty(struct lxc_tty_info *ttys);
 extern int lxc_clear_config_caps(struct lxc_conf *c);
 extern int lxc_clear_config_keepcaps(struct lxc_conf *c);
-extern int lxc_clear_cgroups(struct lxc_conf *c, const char *key);
+extern int lxc_clear_cgroups(struct lxc_conf *c, const char *key, int version);
 extern int lxc_clear_mount_entries(struct lxc_conf *c);
 extern int lxc_clear_automounts(struct lxc_conf *c);
 extern int lxc_clear_hooks(struct lxc_conf *c, const char *key);
@@ -373,14 +400,16 @@ extern void lxc_clear_includes(struct lxc_conf *conf);
 extern int do_rootfs_setup(struct lxc_conf *conf, const char *name,
 			   const char *lxcpath);
 extern int lxc_setup(struct lxc_handler *handler);
+extern int lxc_setup_parent(struct lxc_handler *handler);
 extern int setup_resource_limits(struct lxc_list *limits, pid_t pid);
 extern int find_unmapped_nsid(struct lxc_conf *conf, enum idtype idtype);
 extern int mapped_hostid(unsigned id, struct lxc_conf *conf,
 			 enum idtype idtype);
-extern int chown_mapped_root(char *path, struct lxc_conf *conf);
-extern int lxc_ttys_shift_ids(struct lxc_conf *c);
+extern int chown_mapped_root(const char *path, struct lxc_conf *conf);
 extern int userns_exec_1(struct lxc_conf *conf, int (*fn)(void *), void *data,
 			 const char *fn_name);
+extern int userns_exec_full(struct lxc_conf *conf, int (*fn)(void *),
+			    void *data, const char *fn_name);
 extern int parse_mntopts(const char *mntopts, unsigned long *mntflags,
 			 char **mntdata);
 extern void tmp_proc_unmount(struct lxc_conf *lxc_conf);
@@ -392,5 +421,13 @@ extern unsigned long add_required_remount_flags(const char *s, const char *d,
 						unsigned long flags);
 extern int run_script(const char *name, const char *section, const char *script,
 		      ...);
+extern int run_script_argv(const char *name, unsigned int hook_version,
+			   const char *section, const char *script,
+			   const char *hookname, char **argsin);
+extern int in_caplist(int cap, struct lxc_list *caps);
+extern int setup_sysctl_parameters(struct lxc_list *sysctls);
+extern int lxc_clear_sysctls(struct lxc_conf *c, const char *key);
+extern int setup_proc_filesystem(struct lxc_list *procs, pid_t pid);
+extern int lxc_clear_procs(struct lxc_conf *c, const char *key);
 
 #endif /* __LXC_CONF_H */
