@@ -176,7 +176,7 @@ static void exec_criu(struct criu_opts *opts)
 	FILE *mnts;
 	struct mntent mntent;
 
-	char buf[4096], tty_info[32];
+	char buf[4096], ttys[32];
 	size_t pos;
 
 	/* If we are currently in a cgroup /foo/bar, and the container is in a
@@ -233,12 +233,12 @@ static void exec_criu(struct criu_opts *opts)
 		 */
 		static_args += 6;
 
-		tty_info[0] = 0;
-		if (load_tty_major_minor(opts->user->directory, tty_info, sizeof(tty_info)))
+		ttys[0] = 0;
+		if (load_tty_major_minor(opts->user->directory, ttys, sizeof(ttys)))
 			return;
 
 		/* --inherit-fd fd[%d]:tty[%s] */
-		if (tty_info[0])
+		if (ttys[0])
 			static_args += 2;
 	} else {
 		return;
@@ -493,13 +493,13 @@ static void exec_criu(struct criu_opts *opts)
 		DECLARE_ARG("--restore-detached");
 		DECLARE_ARG("--restore-sibling");
 
-		if (tty_info[0]) {
+		if (ttys[0]) {
 			if (opts->console_fd < 0) {
 				ERROR("lxc.console.path configured on source host but not target");
 				goto err;
 			}
 
-			ret = snprintf(buf, sizeof(buf), "fd[%d]:%s", opts->console_fd, tty_info);
+			ret = snprintf(buf, sizeof(buf), "fd[%d]:%s", opts->console_fd, ttys);
 			if (ret < 0 || ret >= sizeof(buf))
 				goto err;
 
@@ -564,6 +564,8 @@ static void exec_criu(struct criu_opts *opts)
 			switch (n->type) {
 			case LXC_NET_VETH:
 				veth = n->priv.veth_attr.pair;
+				if (veth[0] == '\0')
+					veth = n->priv.veth_attr.veth1;
 
 				if (n->link[0] != '\0') {
 					if (external_not_veth)
@@ -650,6 +652,104 @@ err:
 	free(argv);
 }
 
+/*
+ * Function to check if the checks activated in 'features_to_check' are
+ * available with the current architecture/kernel/criu combination.
+ *
+ * Parameter features_to_check is a bit mask of all features that should be
+ * checked (see feature check defines in lxc/lxccontainer.h).
+ *
+ * If the return value is true, all requested features are supported. If
+ * the return value is false the features_to_check parameter is updated
+ * to reflect which features are available. '0' means no feature but
+ * also that something went totally wrong.
+ *
+ * Some of the code flow of criu_version_ok() is duplicated and maybe it
+ * is a good candidate for refactoring.
+ */
+bool __criu_check_feature(uint64_t *features_to_check)
+{
+	pid_t pid;
+	uint64_t current_bit = 0;
+	int ret;
+	int features = *features_to_check;
+	/* Feature checking is currently always like
+	 * criu check --feature <feature-name>
+	 */
+	char *args[] = { "criu", "check", "--feature", NULL, NULL };
+
+	if ((features & ~FEATURE_MEM_TRACK & ~FEATURE_LAZY_PAGES) != 0) {
+		/* There are feature bits activated we do not understand.
+		 * Refusing to answer at all */
+		*features_to_check = 0;
+		return false;
+	}
+
+	while (current_bit < sizeof(uint64_t) * 8) {
+		/* only test requested features */
+		if (!(features & (1ULL << current_bit))) {
+			/* skip this */
+			current_bit++;
+			continue;
+		}
+
+		pid = fork();
+		if (pid < 0) {
+			SYSERROR("fork() failed");
+			*features_to_check = 0;
+			return false;
+		}
+
+		if (pid == 0) {
+			if ((1ULL << current_bit) == FEATURE_MEM_TRACK)
+				/* This is needed for pre-dump support, which
+				 * enables pre-copy migration. */
+				args[3] = "mem_dirty_track";
+			else if ((1ULL << current_bit) == FEATURE_LAZY_PAGES)
+				/* CRIU has two checks for userfaultfd support.
+				 *
+				 * The simpler check is only for 'uffd'. If the
+				 * kernel supports userfaultfd without noncoop
+				 * then only process can be lazily restored
+				 * which do not fork. With 'uffd-noncoop'
+				 * it is also possible to lazily restore processes
+				 * which do fork. For a container runtime like
+				 * LXC checking only for 'uffd' makes not much sense. */
+				args[3] = "uffd-noncoop";
+			else
+				_exit(EXIT_FAILURE);
+
+			null_stdfds();
+
+			execvp("criu", args);
+			SYSERROR("Failed to exec \"criu\"");
+			_exit(EXIT_FAILURE);
+		}
+
+		ret = wait_for_pid(pid);
+
+		if (ret == -1) {
+			/* It is not known why CRIU failed. Either
+			 * CRIU is not available, the feature check
+			 * does not exist or the feature is not
+			 * supported. */
+			INFO("feature not supported");
+			/* Clear not supported feature bit */
+			features &= ~(1ULL << current_bit);
+		}
+
+		current_bit++;
+		/* no more checks requested; exit check loop */
+		if (!(features & ~((1ULL << current_bit)-1)))
+			break;
+	}
+	if (features != *features_to_check) {
+		*features_to_check = features;
+		return false;
+	}
+	return true;
+}
+
 /*
  * Check to see if the criu version is recent enough for all the features we
  * use. This version allows either CRIU_VERSION or (CRIU_GITID_VERSION and
@@ -685,14 +785,14 @@ static bool criu_version_ok(char **version)
 
 		close(STDERR_FILENO);
 		if (dup2(pipes[1], STDOUT_FILENO) < 0)
-			exit(1);
+			_exit(EXIT_FAILURE);
 
 		path = on_path("criu", NULL);
 		if (!path)
-			exit(1);
+			_exit(EXIT_FAILURE);
 
 		execv(path, args);
-		exit(1);
+		_exit(EXIT_FAILURE);
 	} else {
 		FILE *f;
 		char *tmp;
@@ -823,13 +923,14 @@ out_unlock:
 }
 
 /* do_restore never returns, the calling process is used as the monitor process.
- * do_restore calls exit() if it fails.
+ * do_restore calls _exit() if it fails.
  */
 static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_opts *opts, char *criu_version)
 {
+	int fd, ret;
 	pid_t pid;
 	struct lxc_handler *handler;
-	int status, fd;
+	int status = 0;
 	int pipes[2] = {-1, -1};
 
 	/* Try to detach from the current controlling tty if it exists.
@@ -867,7 +968,11 @@ static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_
 		goto out_fini_handler;
 	}
 
-	resolve_clone_flags(handler);
+	ret = resolve_clone_flags(handler);
+	if (ret < 0) {
+		ERROR("%s - Unsupported clone flag specified", strerror(errno));
+		goto out_fini_handler;
+	}
 
 	if (pipe(pipes) < 0) {
 		SYSERROR("pipe() failed");
@@ -1024,13 +1129,18 @@ static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_
 		 * assign the return here to silence potential.
 		 */
 		ret = snprintf(title, sizeof(title), "[lxc monitor] %s %s", c->config_path, c->name);
+		if (ret < 0 || (size_t)ret >= sizeof(title))
+			INFO("Setting truncated process name");
+
 		ret = setproctitle(title);
+		if (ret < 0)
+			INFO("Failed to set process name");
 
 		ret = lxc_poll(c->name, handler);
 		if (ret)
 			lxc_abort(c->name, handler);
 		lxc_fini(c->name, handler);
-		exit(ret);
+		_exit(ret);
 	}
 
 out_fini_handler:
@@ -1049,13 +1159,13 @@ out:
 		 */
 		if (!status)
 			status = 1;
-		if (write(status_pipe, &status, sizeof(status)) != sizeof(status)) {
+
+		if (write(status_pipe, &status, sizeof(status)) != sizeof(status))
 			SYSERROR("writing status failed");
-		}
 		close(status_pipe);
 	}
 
-	exit(1);
+	_exit(EXIT_FAILURE);
 }
 
 static int save_tty_major_minor(char *directory, struct lxc_container *c, char *tty_id, int len)
@@ -1112,14 +1222,16 @@ static int save_tty_major_minor(char *directory, struct lxc_container *c, char *
 /* do one of either predump or a regular dump */
 static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *opts)
 {
+	int ret;
 	pid_t pid;
-	char *criu_version = NULL;
 	int criuout[2];
+	char *criu_version = NULL;
 
 	if (!criu_ok(c, &criu_version))
 		return false;
 
-	if (pipe(criuout) < 0) {
+	ret = pipe(criuout);
+	if (ret < 0) {
 		SYSERROR("pipe() failed");
 		return false;
 	}
@@ -1139,10 +1251,12 @@ static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *op
 
 		close(criuout[0]);
 
+		lxc_zero_handler(&h);
+
 		h.name = c->name;
 		if (!cgroup_init(&h)) {
 			ERROR("failed to cgroup_init()");
-			exit(1);
+			_exit(EXIT_FAILURE);
 		}
 
 		os.pipefd = criuout[1];
@@ -1152,12 +1266,16 @@ static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *op
 		os.console_name = c->lxc_conf->console.path;
 		os.criu_version = criu_version;
 
-		if (save_tty_major_minor(opts->directory, c, os.tty_id, sizeof(os.tty_id)) < 0)
-			exit(1);
+		ret = save_tty_major_minor(opts->directory, c, os.tty_id, sizeof(os.tty_id));
+		if (ret < 0) {
+			free(criu_version);
+			_exit(EXIT_FAILURE);
+		}
 
 		/* exec_criu() returning is an error */
 		exec_criu(&os);
-		exit(1);
+		free(criu_version);
+		_exit(EXIT_FAILURE);
 	} else {
 		int status;
 		ssize_t n;
@@ -1204,6 +1322,7 @@ fail:
 	close(criuout[0]);
 	close(criuout[1]);
 	rmdir(opts->directory);
+	free(criu_version);
 	return false;
 }
 
