@@ -23,44 +23,31 @@
 #ifndef __LXC_LXCLOCK_H
 #define __LXC_LXCLOCK_H
 
-#include <fcntl.h>
+#include <fcntl.h>           /* For O_* constants */
+#include <sys/stat.h>        /* For mode constants */
+#include <sys/file.h>
 #include <semaphore.h>
 #include <string.h>
-#include <sys/stat.h>
-#include <sys/file.h>
 #include <time.h>
-#include <unistd.h>
-
-#ifndef F_OFD_GETLK
-#define F_OFD_GETLK	36
-#endif
-
-#ifndef F_OFD_SETLK
-#define F_OFD_SETLK	37
-#endif
-
-#ifndef F_OFD_SETLKW
-#define F_OFD_SETLKW	38
-#endif
 
 #define LXC_LOCK_ANON_SEM 1 /*!< Anonymous semaphore lock */
 #define LXC_LOCK_FLOCK    2 /*!< flock(2) lock */
 
-// private
+/* private */
 /*!
  * LXC Lock
 */
 struct lxc_lock {
-	short type; //!< Lock type
+	short type; /*!< Lock type */
 
 	union {
-		sem_t *sem; //!< Anonymous semaphore (LXC_LOCK_ANON_SEM)
+		sem_t *sem; /*!< Anonymous semaphore (LXC_LOCK_ANON_SEM) */
 		/*! LXC_LOCK_FLOCK details */
 		struct {
-			int   fd; //!< fd on which a lock is held (if not -1)
-			char *fname; //!< Name of lock
+			int   fd; /*!< fd on which a lock is held (if not -1) */
+			char *fname; /*!< Name of lock */
 		} f;
-	} u; //!< Container for lock type elements
+	} u; /*!< Container for lock type elements */
 };
 
 /*!
