@@ -204,9 +204,17 @@ static int instantiate_veth(struct lxc_handler *handler, struct lxc_netdev *netd
 	}
 
 	if (netdev->upscript) {
-		err = run_script(handler->name, "net", netdev->upscript, "up",
-				 "veth", veth1, (char*) NULL);
-		if (err)
+		char *argv[] = {
+		    "veth",
+		    netdev->link,
+		    veth1,
+		    NULL,
+		};
+
+		err = run_script_argv(handler->name,
+				handler->conf->hooks_version, "net",
+				netdev->upscript, "up", argv);
+		if (err < 0)
 			goto out_delete;
 	}
 
@@ -254,9 +262,16 @@ static int instantiate_macvlan(struct lxc_handler *handler, struct lxc_netdev *n
 	}
 
 	if (netdev->upscript) {
-		err = run_script(handler->name, "net", netdev->upscript, "up",
-				 "macvlan", netdev->link, (char*) NULL);
-		if (err)
+		char *argv[] = {
+		    "macvlan",
+		    netdev->link,
+		    NULL,
+		};
+
+		err = run_script_argv(handler->name,
+				handler->conf->hooks_version, "net",
+				netdev->upscript, "up", argv);
+		if (err < 0)
 			goto on_error;
 	}
 
@@ -323,6 +338,13 @@ static int instantiate_vlan(struct lxc_handler *handler, struct lxc_netdev *netd
 
 static int instantiate_phys(struct lxc_handler *handler, struct lxc_netdev *netdev)
 {
+	int ret;
+	char *argv[] = {
+		"phys",
+		netdev->link,
+		NULL,
+	};
+
 	if (netdev->link[0] == '\0') {
 		ERROR("No link for physical interface specified");
 		return -1;
@@ -346,27 +368,34 @@ static int instantiate_phys(struct lxc_handler *handler, struct lxc_netdev *netd
 	 */
 	netdev->priv.phys_attr.ifindex = netdev->ifindex;
 
-	if (netdev->upscript) {
-		int err;
-		err = run_script(handler->name, "net", netdev->upscript,
-				 "up", "phys", netdev->link, (char*) NULL);
-		if (err)
-			return -1;
-	}
+	if (!netdev->upscript)
+		return 0;
+
+	ret = run_script_argv(handler->name, handler->conf->hooks_version,
+			      "net", netdev->upscript, "up", argv);
+	if (ret < 0)
+		return -1;
 
 	return 0;
 }
 
 static int instantiate_empty(struct lxc_handler *handler, struct lxc_netdev *netdev)
 {
+	int ret;
+	char *argv[] = {
+	    "empty",
+	    NULL,
+	};
+
 	netdev->ifindex = 0;
-	if (netdev->upscript) {
-		int err;
-		err = run_script(handler->name, "net", netdev->upscript,
-				 "up", "empty", (char*) NULL);
-		if (err)
-			return -1;
-	}
+	if (!netdev->upscript)
+		return 0;
+
+	ret = run_script_argv(handler->name, handler->conf->hooks_version,
+			      "net", netdev->upscript, "up", argv);
+	if (ret < 0)
+		return -1;
+
 	return 0;
 }
 
@@ -387,34 +416,48 @@ static  instantiate_cb netdev_conf[LXC_NET_MAXCONFTYPE + 1] = {
 
 static int shutdown_veth(struct lxc_handler *handler, struct lxc_netdev *netdev)
 {
-	char *veth1;
-	int err;
+	int ret;
+	char *argv[] = {
+	    "veth",
+	    netdev->link,
+	    NULL,
+	    NULL,
+	};
+
+	if (!netdev->downscript)
+		return 0;
 
 	if (netdev->priv.veth_attr.pair[0] != '\0')
-		veth1 = netdev->priv.veth_attr.pair;
+		argv[2] = netdev->priv.veth_attr.pair;
 	else
-		veth1 = netdev->priv.veth_attr.veth1;
+		argv[2] = netdev->priv.veth_attr.veth1;
+
+	ret = run_script_argv(handler->name,
+			handler->conf->hooks_version, "net",
+			netdev->downscript, "down", argv);
+	if (ret < 0)
+		return -1;
 
-	if (netdev->downscript) {
-		err = run_script(handler->name, "net", netdev->downscript,
-				 "down", "veth", veth1, (char*) NULL);
-		if (err)
-			return -1;
-	}
 	return 0;
 }
 
 static int shutdown_macvlan(struct lxc_handler *handler, struct lxc_netdev *netdev)
 {
-	int err;
+	int ret;
+	char *argv[] = {
+		"macvlan",
+		netdev->link,
+		NULL,
+	};
+
+	if (!netdev->downscript)
+		return 0;
+
+	ret = run_script_argv(handler->name, handler->conf->hooks_version,
+			      "net", netdev->downscript, "down", argv);
+	if (ret < 0)
+		return -1;
 
-	if (netdev->downscript) {
-		err = run_script(handler->name, "net", netdev->downscript,
-				 "down", "macvlan", netdev->link,
-				 (char*) NULL);
-		if (err)
-			return -1;
-	}
 	return 0;
 }
 
@@ -425,27 +468,40 @@ static int shutdown_vlan(struct lxc_handler *handler, struct lxc_netdev *netdev)
 
 static int shutdown_phys(struct lxc_handler *handler, struct lxc_netdev *netdev)
 {
-	int err;
+	int ret;
+	char *argv[] = {
+	    "phys",
+	    netdev->link,
+	    NULL,
+	};
+
+	if (!netdev->downscript)
+		return 0;
+
+	ret = run_script_argv(handler->name, handler->conf->hooks_version,
+			      "net", netdev->downscript, "down", argv);
+	if (ret < 0)
+		return -1;
 
-	if (netdev->downscript) {
-		err = run_script(handler->name, "net", netdev->downscript,
-				 "down", "phys", netdev->link, (char*) NULL);
-		if (err)
-			return -1;
-	}
 	return 0;
 }
 
 static int shutdown_empty(struct lxc_handler *handler, struct lxc_netdev *netdev)
 {
-	int err;
+	int ret;
+	char *argv[] = {
+	    "empty",
+	    NULL,
+	};
+
+	if (!netdev->downscript)
+		return 0;
+
+	ret = run_script_argv(handler->name, handler->conf->hooks_version,
+			      "net", netdev->downscript, "down", argv);
+	if (ret < 0)
+		return -1;
 
-	if (netdev->downscript) {
-		err = run_script(handler->name, "net", netdev->downscript,
-				 "down", "empty", (char*) NULL);
-		if (err)
-			return -1;
-	}
 	return 0;
 }
 
@@ -579,7 +635,7 @@ static int lxc_netdev_rename_by_name_in_netns(pid_t pid, const char *old,
 	if (!switch_to_ns(pid, "net"))
 		return -1;
 
-	exit(lxc_netdev_rename_by_name(old, new));
+	_exit(lxc_netdev_rename_by_name(old, new));
 }
 
 static int lxc_netdev_move_wlan(char *physname, const char *ifname, pid_t pid,
@@ -607,7 +663,7 @@ static int lxc_netdev_move_wlan(char *physname, const char *ifname, pid_t pid,
 		sprintf(pidstr, "%d", pid);
 		execlp("iw", "iw", "phy", physname, "set", "netns", pidstr,
 		       (char *)NULL);
-		exit(EXIT_FAILURE);
+		_exit(EXIT_FAILURE);
 	}
 
 	if (wait_for_pid(fpid))
@@ -935,7 +991,6 @@ int netdev_get_mtu(int ifindex)
 			goto out;
 
 		recv_len = err;
-		err = 0;
 
 		/* Satisfy the typing for the netlink macros */
 		msg = answer->nlmsghdr;
@@ -1313,32 +1368,6 @@ static int proc_sys_net_write(const char *path, const char *value)
 	return err;
 }
 
-static int ip_forward_set(const char *ifname, int family, int flag)
-{
-	int rc;
-	char path[MAXPATHLEN];
-
-	if (family != AF_INET && family != AF_INET6)
-		return -EINVAL;
-
-	rc = snprintf(path, MAXPATHLEN, "/proc/sys/net/%s/conf/%s/forwarding",
-		      family == AF_INET ? "ipv4" : "ipv6", ifname);
-	if (rc < 0 || (size_t)rc >= MAXPATHLEN)
-		return -E2BIG;
-
-	return proc_sys_net_write(path, flag ? "1" : "0");
-}
-
-int lxc_ip_forward_on(const char *ifname, int family)
-{
-	return ip_forward_set(ifname, family, 1);
-}
-
-int lxc_ip_forward_off(const char *ifname, int family)
-{
-	return ip_forward_set(ifname, family, 0);
-}
-
 static int neigh_proxy_set(const char *ifname, int family, int flag)
 {
 	int ret;
@@ -1377,7 +1406,6 @@ int lxc_convert_mac(char *macaddr, struct sockaddr *sockaddr)
 	data = (unsigned char *)sockaddr->sa_data;
 
 	while ((*macaddr != '\0') && (i < ETH_ALEN)) {
-		val = 0;
 		c = *macaddr++;
 		if (isdigit(c))
 			val = c - '0';
@@ -1917,6 +1945,7 @@ static const char padchar[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 char *lxc_mkifname(char *template)
 {
+	int ret;
 	unsigned int seed;
 	FILE *urandom;
 	struct ifaddrs *ifa, *ifaddr;
@@ -1928,7 +1957,11 @@ char *lxc_mkifname(char *template)
 		return NULL;
 
 	/* Get all the network interfaces. */
-	getifaddrs(&ifaddr);
+	ret = getifaddrs(&ifaddr);
+	if (ret < 0) {
+		ERROR("%s - Failed to get network interfaces", strerror(errno));
+		return NULL;
+	}
 
 	/* Initialize the random number generator. */
 	urandom = fopen("/dev/urandom", "r");
@@ -2052,7 +2085,7 @@ int lxc_find_gateway_addresses(struct lxc_handler *handler)
 }
 
 #define LXC_USERNIC_PATH LIBEXECDIR "/lxc/lxc-user-nic"
-static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
+static int lxc_create_network_unpriv_exec(const char *lxcpath, const char *lxcname,
 					  struct lxc_netdev *netdev, pid_t pid)
 {
 	int ret;
@@ -2093,7 +2126,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		close(pipefd[1]);
 		if (ret < 0) {
 			SYSERROR("Failed to duplicate std{err,out} file descriptor");
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		if (netdev->link[0] != '\0')
@@ -2103,7 +2136,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 
 		ret = snprintf(pidstr, LXC_NUMSTRLEN64, "%d", pid);
 		if (ret < 0 || ret >= LXC_NUMSTRLEN64)
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		pidstr[LXC_NUMSTRLEN64 - 1] = '\0';
 
 		INFO("Execing lxc-user-nic create %s %s %s veth %s %s", lxcpath,
@@ -2118,7 +2151,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 			       lxcpath, lxcname, pidstr, "veth", netdev_link,
 			       (char *)NULL);
 		SYSERROR("Failed to execute lxc-user-nic");
-		exit(EXIT_FAILURE);
+		_exit(EXIT_FAILURE);
 	}
 
 	/* close the write-end of the pipe */
@@ -2128,13 +2161,13 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 	if (bytes < 0) {
 		SYSERROR("Failed to read from pipe file descriptor");
 		close(pipefd[0]);
-		return -1;
+	} else {
+		buffer[bytes - 1] = '\0';
 	}
-	buffer[bytes - 1] = '\0';
 
 	ret = wait_for_pid(child);
 	close(pipefd[0]);
-	if (ret != 0) {
+	if (ret != 0 || bytes < 0) {
 		ERROR("lxc-user-nic failed to configure requested network: %s",
 		      buffer[0] != '\0' ? buffer : "(null)");
 		return -1;
@@ -2196,7 +2229,7 @@ static int lxc_create_network_unpriv_exec(const char *lxcpath, char *lxcname,
 	return 0;
 }
 
-static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
+static int lxc_delete_network_unpriv_exec(const char *lxcpath, const char *lxcname,
 					  struct lxc_netdev *netdev,
 					  const char *netns_path)
 {
@@ -2236,7 +2269,7 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		close(pipefd[1]);
 		if (ret < 0) {
 			SYSERROR("Failed to duplicate std{err,out} file descriptor");
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		if (netdev->priv.veth_attr.pair[0] != '\0')
@@ -2245,13 +2278,13 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
 			hostveth = netdev->priv.veth_attr.veth1;
 		if (hostveth[0] == '\0') {
 			SYSERROR("Host side veth device name is missing");
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		if (netdev->link[0] == '\0') {
 			SYSERROR("Network link for network device \"%s\" is "
 				 "missing", netdev->priv.veth_attr.veth1);
-			exit(EXIT_FAILURE);
+			_exit(EXIT_FAILURE);
 		}
 
 		INFO("Execing lxc-user-nic delete %s %s %s veth %s %s", lxcpath,
@@ -2260,7 +2293,7 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
 		       lxcname, netns_path, "veth", netdev->link, hostveth,
 		       (char *)NULL);
 		SYSERROR("Failed to exec lxc-user-nic.");
-		exit(EXIT_FAILURE);
+		_exit(EXIT_FAILURE);
 	}
 
 	close(pipefd[1]);
@@ -2269,19 +2302,18 @@ static int lxc_delete_network_unpriv_exec(const char *lxcpath, char *lxcname,
 	if (bytes < 0) {
 		SYSERROR("Failed to read from pipe file descriptor.");
 		close(pipefd[0]);
-		return -1;
+	} else {
+		buffer[bytes - 1] = '\0';
 	}
-	buffer[bytes - 1] = '\0';
 
-	if (wait_for_pid(child) != 0) {
+	ret = wait_for_pid(child);
+	close(pipefd[0]);
+	if (ret != 0 || bytes < 0) {
 		ERROR("lxc-user-nic failed to delete requested network: %s",
 		      buffer[0] != '\0' ? buffer : "(null)");
-		close(pipefd[0]);
 		return -1;
 	}
 
-	close(pipefd[0]);
-
 	return 0;
 }
 
@@ -2301,21 +2333,17 @@ bool lxc_delete_network_unpriv(struct lxc_handler *handler)
 	 * \0
 	 */
 	char netns_path[6 + LXC_NUMSTRLEN64 + 4 + LXC_NUMSTRLEN64 + 1];
-	bool deleted_all = true;
-
-	if (handler->am_root)
-		return true;
 
 	*netns_path = '\0';
 
-	if (handler->netnsfd < 0) {
+	if (handler->nsfd[LXC_NS_NET] < 0) {
 		DEBUG("Cannot not guarantee safe deletion of network devices. "
 		      "Manual cleanup maybe needed");
 		return false;
 	}
 
 	ret = snprintf(netns_path, sizeof(netns_path), "/proc/%d/fd/%d",
-		       getpid(), handler->netnsfd);
+		       lxc_raw_getpid(), handler->nsfd[LXC_NS_NET]);
 	if (ret < 0 || ret >= sizeof(netns_path))
 		return false;
 
@@ -2340,7 +2368,7 @@ bool lxc_delete_network_unpriv(struct lxc_handler *handler)
 				TRACE("Renamed interface with index %d to its "
 				      "initial name \"%s\"",
 				      netdev->ifindex, netdev->link);
-			continue;
+			goto clear_ifindices;
 		}
 
 		ret = netdev_deconf[netdev->type](handler, netdev);
@@ -2348,32 +2376,44 @@ bool lxc_delete_network_unpriv(struct lxc_handler *handler)
 			WARN("Failed to deconfigure network device");
 
 		if (netdev->type != LXC_NET_VETH)
-			continue;
+			goto clear_ifindices;
 
-		if (!is_ovs_bridge(netdev->link))
-			continue;
+		if (netdev->link[0] == '\0' || !is_ovs_bridge(netdev->link))
+			goto clear_ifindices;
 
 		if (netdev->priv.veth_attr.pair[0] != '\0')
 			hostveth = netdev->priv.veth_attr.pair;
 		else
 			hostveth = netdev->priv.veth_attr.veth1;
 		if (hostveth[0] == '\0')
-			continue;
+			goto clear_ifindices;
 
 		ret = lxc_delete_network_unpriv_exec(handler->lxcpath,
 						     handler->name, netdev,
 						     netns_path);
 		if (ret < 0) {
-			deleted_all = false;
 			WARN("Failed to remove port \"%s\" from openvswitch "
 			     "bridge \"%s\"", hostveth, netdev->link);
-			continue;
+			goto clear_ifindices;
 		}
 		INFO("Removed interface \"%s\" from \"%s\"", hostveth,
 		     netdev->link);
+
+clear_ifindices:
+		/* We need to clear any ifindeces we recorded so liblxc won't
+		 * have cached stale data which would cause it to fail on reboot
+		 * we're we don't re-read the on-disk config file.
+		 */
+		netdev->ifindex = 0;
+		if (netdev->type == LXC_NET_PHYS) {
+			netdev->priv.phys_attr.ifindex = 0;
+		} else if (netdev->type == LXC_NET_VETH) {
+			netdev->priv.veth_attr.veth1[0] = '\0';
+			netdev->priv.veth_attr.ifindex = 0;
+		}
 	}
 
-	return deleted_all;
+	return true;
 }
 
 int lxc_create_network_priv(struct lxc_handler *handler)
@@ -2402,14 +2442,14 @@ int lxc_create_network_priv(struct lxc_handler *handler)
 	return 0;
 }
 
-int lxc_network_move_created_netdev_priv(const char *lxcpath, char *lxcname,
+int lxc_network_move_created_netdev_priv(const char *lxcpath, const char *lxcname,
 					 struct lxc_list *network, pid_t pid)
 {
 	int ret;
 	char ifname[IFNAMSIZ];
 	struct lxc_list *iterator;
 
-	if (am_unpriv())
+	if (am_guest_unpriv())
 		return 0;
 
 	lxc_list_for_each(iterator, network) {
@@ -2442,12 +2482,12 @@ int lxc_network_move_created_netdev_priv(const char *lxcpath, char *lxcname,
 	return 0;
 }
 
-int lxc_create_network_unpriv(const char *lxcpath, char *lxcname,
+int lxc_create_network_unpriv(const char *lxcpath, const char *lxcname,
 			      struct lxc_list *network, pid_t pid)
 {
 	struct lxc_list *iterator;
 
-	if (!am_unpriv())
+	if (!am_guest_unpriv())
 		return 0;
 
 	lxc_list_for_each(iterator, network) {
@@ -2481,10 +2521,6 @@ bool lxc_delete_network_priv(struct lxc_handler *handler)
 	int ret;
 	struct lxc_list *iterator;
 	struct lxc_list *network = &handler->conf->network;
-	bool deleted_all = true;
-
-	if (!handler->am_root)
-		return true;
 
 	lxc_list_for_each(iterator, network) {
 		char *hostveth = NULL;
@@ -2507,7 +2543,7 @@ bool lxc_delete_network_priv(struct lxc_handler *handler)
 				      "\"%s\" to its initial name \"%s\"",
 				      netdev->ifindex, netdev->name,
 				      netdev->link);
-			continue;
+			goto clear_ifindices;
 		}
 
 		ret = netdev_deconf[netdev->type](handler, netdev);
@@ -2521,24 +2557,23 @@ bool lxc_delete_network_priv(struct lxc_handler *handler)
 		ret = lxc_netdev_delete_by_index(netdev->ifindex);
 		if (-ret == ENODEV) {
 			INFO("Interface \"%s\" with index %d already "
-					"deleted or existing in different network "
-					"namespace",
-					netdev->name[0] != '\0' ? netdev->name : "(null)",
-					netdev->ifindex);
+			     "deleted or existing in different network "
+			     "namespace",
+			     netdev->name[0] != '\0' ? netdev->name : "(null)",
+			     netdev->ifindex);
 		} else if (ret < 0) {
-			deleted_all = false;
 			WARN("Failed to remove interface \"%s\" with "
-					"index %d: %s",
-					netdev->name[0] != '\0' ? netdev->name : "(null)",
-					netdev->ifindex, strerror(-ret));
-			continue;
+			     "index %d: %s",
+			     netdev->name[0] != '\0' ? netdev->name : "(null)",
+			     netdev->ifindex, strerror(-ret));
+			goto clear_ifindices;
 		}
 		INFO("Removed interface \"%s\" with index %d",
-				netdev->name[0] != '\0' ? netdev->name : "(null)",
-				netdev->ifindex);
+		     netdev->name[0] != '\0' ? netdev->name : "(null)",
+		     netdev->ifindex);
 
 		if (netdev->type != LXC_NET_VETH)
-			continue;
+			goto clear_ifindices;
 
 		/* Explicitly delete host veth device to prevent lingering
 		 * devices. We had issues in LXD around this.
@@ -2548,20 +2583,21 @@ bool lxc_delete_network_priv(struct lxc_handler *handler)
 		else
 			hostveth = netdev->priv.veth_attr.veth1;
 		if (hostveth[0] == '\0')
-			continue;
+			goto clear_ifindices;
 
 		ret = lxc_netdev_delete_by_name(hostveth);
 		if (ret < 0) {
-			deleted_all = false;
 			WARN("Failed to remove interface \"%s\" from \"%s\": %s",
 			     hostveth, netdev->link, strerror(-ret));
-			continue;
+			goto clear_ifindices;
 		}
 		INFO("Removed interface \"%s\" from \"%s\"", hostveth, netdev->link);
 
-		if (!is_ovs_bridge(netdev->link)) {
+		if (netdev->link[0] == '\0' || !is_ovs_bridge(netdev->link)) {
 			netdev->priv.veth_attr.veth1[0] = '\0';
-			continue;
+			netdev->ifindex = 0;
+			netdev->priv.veth_attr.ifindex = 0;
+			goto clear_ifindices;
 		}
 
 		/* Delete the openvswitch port. */
@@ -2573,10 +2609,21 @@ bool lxc_delete_network_priv(struct lxc_handler *handler)
 			INFO("Removed port \"%s\" from openvswitch bridge \"%s\"",
 			     hostveth, netdev->link);
 
-		netdev->priv.veth_attr.veth1[0] = '\0';
+clear_ifindices:
+		/* We need to clear any ifindeces we recorded so liblxc won't
+		 * have cached stale data which would cause it to fail on reboot
+		 * we're we don't re-read the on-disk config file.
+		 */
+		netdev->ifindex = 0;
+		if (netdev->type == LXC_NET_PHYS) {
+			netdev->priv.phys_attr.ifindex = 0;
+		} else if (netdev->type == LXC_NET_VETH) {
+			netdev->priv.veth_attr.veth1[0] = '\0';
+			netdev->priv.veth_attr.ifindex = 0;
+		}
 	}
 
-	return deleted_all;
+	return true;
 }
 
 int lxc_requests_empty_network(struct lxc_handler *handler)
@@ -2608,7 +2655,7 @@ int lxc_restore_phys_nics_to_netns(struct lxc_handler *handler)
 	int oldfd;
 	char ifname[IFNAMSIZ];
 	struct lxc_list *iterator;
-	int netnsfd = handler->netnsfd;
+	int netnsfd = handler->nsfd[LXC_NS_NET];
 	struct lxc_conf *conf = handler->conf;
 
 	/* We need CAP_NET_ADMIN in the parent namespace in order to setns() to
@@ -2620,7 +2667,7 @@ int lxc_restore_phys_nics_to_netns(struct lxc_handler *handler)
 
 	TRACE("Moving physical network devices back to parent network namespace");
 
-	oldfd = lxc_preserve_ns(getpid(), "net");
+	oldfd = lxc_preserve_ns(lxc_raw_getpid(), "net");
 	if (oldfd < 0) {
 		SYSERROR("Failed to preserve network namespace");
 		return -1;
@@ -2967,13 +3014,6 @@ int lxc_setup_network_in_child_namespaces(const struct lxc_conf *conf,
 	lxc_list_for_each(iterator, network) {
 		netdev = iterator->elem;
 
-		/* REMOVE in LXC 3.0 */
-		if (netdev->idx < 0) {
-			ERROR("WARNING: using \"lxc.network.*\" keys to define "
-			      "networks is DEPRECATED, please switch to using "
-			      "\"lxc.net.[i].* keys\"");
-		}
-
 		if (lxc_setup_netdev_in_child_namespaces(netdev)) {
 			ERROR("failed to setup netdev");
 			return -1;
@@ -3096,3 +3136,17 @@ int lxc_network_recv_name_and_ifindex_from_child(struct lxc_handler *handler)
 
 	return 0;
 }
+
+void lxc_delete_network(struct lxc_handler *handler)
+{
+	bool bret;
+
+	if (handler->am_root)
+		bret = lxc_delete_network_priv(handler);
+	else
+		bret = lxc_delete_network_unpriv(handler);
+	if (!bret)
+		DEBUG("Failed to delete network devices");
+	else
+		DEBUG("Deleted network devices");
+}
