@@ -187,7 +187,8 @@ bool btrfs_detect(const char *path)
 int btrfs_mount(struct lxc_storage *bdev)
 {
 	unsigned long mntflags;
-	char *mntdata, *src;
+	char *mntdata;
+	const char *src;
 	int ret;
 
 	if (strcmp(bdev->type, "btrfs"))
@@ -348,7 +349,7 @@ out:
 
 int btrfs_snapshot_wrapper(void *data)
 {
-	char *src;
+	const char *src;
 	struct rsync_data_char *arg = data;
 
 	if (setgid(0) < 0) {
@@ -372,7 +373,7 @@ int btrfs_clonepaths(struct lxc_storage *orig, struct lxc_storage *new,
 		     const char *oldpath, const char *lxcpath, int snap,
 		     uint64_t newsize, struct lxc_conf *conf)
 {
-	char *src;
+	const char *src;
 
 	if (!orig->dest || !orig->src)
 		return -1;
@@ -433,9 +434,9 @@ bool btrfs_create_clone(struct lxc_conf *conf, struct lxc_storage *orig,
 	/* rsync the contents from source to target */
 	data.orig = orig;
 	data.new = new;
-	if (am_unpriv()) {
-		ret = userns_exec_1(conf, lxc_storage_rsync_exec_wrapper, &data,
-				    "lxc_storage_rsync_exec_wrapper");
+	if (am_guest_unpriv()) {
+		ret = userns_exec_full(conf, lxc_storage_rsync_exec_wrapper,
+				       &data, "lxc_storage_rsync_exec_wrapper");
 		if (ret < 0) {
 			ERROR("Failed to rsync from \"%s\" into \"%s\"",
 			      orig->dest, new->dest);
@@ -465,14 +466,14 @@ bool btrfs_create_snapshot(struct lxc_conf *conf, struct lxc_storage *orig,
 	if (ret < 0 && errno != ENOENT)
 		return false;
 
-	if (am_unpriv()) {
+	if (am_guest_unpriv()) {
 		struct rsync_data_char args;
 
-		args.src = orig->dest;
+		args.src = orig->src;
 		args.dest = new->dest;
 
 		ret = userns_exec_1(conf, btrfs_snapshot_wrapper, &args,
-				"btrfs_snapshot_wrapper");
+				    "btrfs_snapshot_wrapper");
 		if (ret < 0) {
 			ERROR("Failed to run \"btrfs_snapshot_wrapper\"");
 			return false;
@@ -483,7 +484,7 @@ bool btrfs_create_snapshot(struct lxc_conf *conf, struct lxc_storage *orig,
 		return true;
 	}
 
-	ret = btrfs_snapshot(orig->dest, new->dest);
+	ret = btrfs_snapshot(orig->src, new->dest);
 	if (ret < 0) {
 		SYSERROR("Failed to create btrfs snapshot \"%s\" from \"%s\"",
 			 new->dest, orig->dest);
@@ -821,7 +822,7 @@ bool btrfs_try_remove_subvol(const char *path)
 
 int btrfs_destroy(struct lxc_storage *orig)
 {
-	char *src;
+	const char *src;
 
 	src = lxc_storage_get_path(orig->src, "btrfs");
 
