@@ -95,7 +95,7 @@ struct btrfs_ioctl_vol_args_v2 {
 	union {
 		struct {
 			unsigned long long size;
-			//struct btrfs_qgroup_inherit *qgroup_inherit;
+			/*struct btrfs_qgroup_inherit *qgroup_inherit; */
 			void *qgroup_inherit;
 		};
 		unsigned long long unused[4];
@@ -385,18 +385,18 @@ struct my_btrfs_tree {
 };
 
 extern int btrfs_clonepaths(struct lxc_storage *orig, struct lxc_storage *new,
-                           const char *oldname, const char *cname,
-                           const char *oldpath, const char *lxcpath, int snap,
-                           uint64_t newsize, struct lxc_conf *conf);
+			    const char *oldname, const char *cname,
+			    const char *oldpath, const char *lxcpath, int snap,
+			    uint64_t newsize, struct lxc_conf *conf);
 extern int btrfs_create(struct lxc_storage *bdev, const char *dest,
-                       const char *n, struct bdev_specs *specs);
+			const char *n, struct bdev_specs *specs);
 extern int btrfs_destroy(struct lxc_storage *orig);
-extern int btrfs_detect(const char *path);
+extern bool btrfs_detect(const char *path);
 extern int btrfs_mount(struct lxc_storage *bdev);
 extern int btrfs_umount(struct lxc_storage *bdev);
 
 extern char *get_btrfs_subvol_path(int fd, u64 dir_id, u64 objid, char *name,
-				   u16 name_len);
+				   int name_len);
 extern int btrfs_list_get_path_rootid(int fd, u64 *treeid);
 extern bool is_btrfs_fs(const char *path);
 extern int is_btrfs_subvol(const char *path);
@@ -405,9 +405,9 @@ extern int btrfs_same_fs(const char *orig, const char *new);
 extern int btrfs_snapshot(const char *orig, const char *new);
 extern int btrfs_snapshot_wrapper(void *data);
 extern bool btrfs_create_clone(struct lxc_conf *conf, struct lxc_storage *orig,
-                              struct lxc_storage *new, uint64_t newsize);
+			       struct lxc_storage *new, uint64_t newsize);
 extern bool btrfs_create_snapshot(struct lxc_conf *conf,
-                                 struct lxc_storage *orig,
-                                 struct lxc_storage *new, uint64_t newsize);
+				  struct lxc_storage *orig,
+				  struct lxc_storage *new, uint64_t newsize);
 
 #endif /* __LXC_BTRFS_H */
