@@ -54,7 +54,7 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 		   int snap, uint64_t newsize, struct lxc_conf *conf)
 {
 	int ret;
-	char *src;
+	const char *src;
 
 	if (!snap) {
 		ERROR("The overlay storage driver can only be used for "
@@ -73,7 +73,7 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 		return -1;
 	}
 
-	if (am_unpriv()) {
+	if (am_guest_unpriv()) {
 		ret = chown_mapped_root(new->dest, conf);
 		if (ret < 0)
 			WARN("Failed to update ownership of %s", new->dest);
@@ -109,9 +109,9 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 			return -1;
 		}
 
-		strncpy(delta, new->dest, lastslashidx + 1);
-		strncpy(delta + lastslashidx, "delta0", sizeof("delta0") - 1);
-		delta[lastslashidx + sizeof("delta0")] = '\0';
+		memcpy(delta, new->dest, lastslashidx + 1);
+		memcpy(delta + lastslashidx, "delta0", sizeof("delta0") - 1);
+		delta[lastslashidx + sizeof("delta0") - 1] = '\0';
 
 		ret = mkdir(delta, 0755);
 		if (ret < 0 && errno != EEXIST) {
@@ -120,7 +120,7 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 			return -1;
 		}
 
-		if (am_unpriv()) {
+		if (am_guest_unpriv()) {
 			ret = chown_mapped_root(delta, conf);
 			if (ret < 0)
 				WARN("Failed to update ownership of %s", delta);
@@ -141,18 +141,19 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 			return -1;
 		}
 
-		strncpy(work, new->dest, lastslashidx + 1);
-		strncpy(work + lastslashidx, "olwork", sizeof("olwork") - 1);
-		work[lastslashidx + sizeof("olwork")] = '\0';
+		memcpy(work, new->dest, lastslashidx + 1);
+		memcpy(work + lastslashidx, "olwork", sizeof("olwork") - 1);
+		work[lastslashidx + sizeof("olwork") - 1] = '\0';
 
-		if (mkdir(work, 0755) < 0) {
-			SYSERROR("error: mkdir %s", work);
+		ret = mkdir(work, 0755);
+		if (ret < 0) {
+			SYSERROR("Failed to create directory \"%s\"", work);
 			free(delta);
 			free(work);
 			return -1;
 		}
 
-		if (am_unpriv()) {
+		if (am_guest_unpriv()) {
 			ret = chown_mapped_root(work, conf);
 			if (ret < 0)
 				WARN("Failed to update ownership of %s", work);
@@ -198,18 +199,16 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 			return -22;
 		}
 
-		nsrc = strchr(osrc, ':') + 1;
-		if ((nsrc != osrc + 8) && (nsrc != osrc + 10)) {
-			free(osrc);
-			ERROR("Detected \":\" in \"%s\" at wrong position",
-			      osrc);
-			return -22;
-		}
+		nsrc = osrc;
+		if (strncmp(osrc, "overlay:", 8) == 0)
+			nsrc += 8;
+		else if (strncmp(osrc, "overlayfs:", 10) == 0)
+			nsrc += 10;
 
 		odelta = strchr(nsrc, ':');
 		if (!odelta) {
-			free(osrc);
 			ERROR("Failed to find \":\" in \"%s\"", nsrc);
+			free(osrc);
 			return -22;
 		}
 
@@ -219,13 +218,13 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 
 		ret = mkdir(ndelta, 0755);
 		if (ret < 0 && errno != EEXIST) {
+			SYSERROR("Failed to create directory \"%s\"", ndelta);
 			free(osrc);
 			free(ndelta);
-			SYSERROR("Failed to create directory \"%s\"", ndelta);
 			return -1;
 		}
 
-		if (am_unpriv()) {
+		if (am_guest_unpriv()) {
 			ret = chown_mapped_root(ndelta, conf);
 			if (ret < 0)
 				WARN("Failed to update ownership of %s",
@@ -237,9 +236,9 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 		 */
 		lastslash = strrchr(ndelta, '/');
 		if (!lastslash) {
+			ERROR("Failed to detect \"/\" in \"%s\"", ndelta);
 			free(osrc);
 			free(ndelta);
-			ERROR("Failed to detect \"/\" in \"%s\"", ndelta);
 			return -1;
 		}
 		lastslash++;
@@ -253,20 +252,20 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 			return -1;
 		}
 
-		strncpy(work, ndelta, lastslashidx + 1);
-		strncpy(work + lastslashidx, "olwork", sizeof("olwork") - 1);
-		work[lastslashidx + sizeof("olwork")] = '\0';
+		memcpy(work, ndelta, lastslashidx + 1);
+		memcpy(work + lastslashidx, "olwork", sizeof("olwork") - 1);
+		work[lastslashidx + sizeof("olwork") - 1] = '\0';
 
 		ret = mkdir(work, 0755);
 		if (ret < 0 && errno != EEXIST) {
+			SYSERROR("Failed to create directory \"%s\"", ndelta);
 			free(osrc);
 			free(ndelta);
 			free(work);
-			SYSERROR("Failed to create directory \"%s\"", ndelta);
 			return -1;
 		}
 
-		if (am_unpriv()) {
+		if (am_guest_unpriv()) {
 			ret = chown_mapped_root(work, conf);
 			if (ret < 0)
 				WARN("Failed to update ownership of %s", work);
@@ -322,7 +321,7 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 
 		s1 = strrchr(clean_old_path, '/');
 		if (!s1) {
-			ERROR("Failed to detect \"/\" in string \"%s\"", s1);
+			ERROR("Failed to detect \"/\" in string \"%s\"", clean_old_path);
 			free(clean_old_path);
 			free(clean_new_path);
 			return -1;
@@ -330,7 +329,7 @@ int ovl_clonepaths(struct lxc_storage *orig, struct lxc_storage *new, const char
 
 		s2 = strrchr(clean_new_path, '/');
 		if (!s2) {
-			ERROR("Failed to detect \"/\" in string \"%s\"", s2);
+			ERROR("Failed to detect \"/\" in string \"%s\"", clean_new_path);
 			free(clean_old_path);
 			free(clean_new_path);
 			return -1;
@@ -411,15 +410,12 @@ int ovl_create(struct lxc_storage *bdev, const char *dest, const char *n,
 		return -1;
 	}
 
-	delta = malloc(len + 1);
+	delta = strdup(dest);
 	if (!delta) {
 		ERROR("Failed to allocate memory");
 		return -1;
 	}
-
-	strncpy(delta, dest, len);
-	strncpy(delta + len - 6, "delta0", sizeof("delta0") - 1);
-	delta[len + sizeof("delta0")] = '\0';
+	memcpy(delta + len - 6, "delta0", sizeof("delta0") - 1);
 
 	ret = mkdir_p(delta, 0755);
 	if (ret < 0) {
@@ -457,22 +453,17 @@ int ovl_create(struct lxc_storage *bdev, const char *dest, const char *n,
 
 int ovl_destroy(struct lxc_storage *orig)
 {
-	bool ovl;
 	char *upper = orig->src;
 
-	ovl = !strncmp(upper, "overlay:", 8);
-	if (!ovl && strncmp(upper, "overlayfs:", 10))
-		return -22;
-
 	/* For an overlay container the rootfs is considered immutable
 	 * and cannot be removed when restoring from a snapshot.
 	 */
 	if (orig->flags & LXC_STORAGE_INTERNAL_OVERLAY_RESTORE)
 		return 0;
 
-	if (ovl)
+	if (strncmp(upper, "overlay:", 8) == 0)
 		upper += 8;
-	else
+	else if (strncmp(upper, "overlayfs:", 10) == 0)
 		upper += 10;
 
 	upper = strchr(upper, ':');
@@ -485,10 +476,10 @@ int ovl_destroy(struct lxc_storage *orig)
 
 bool ovl_detect(const char *path)
 {
-	if (!strncmp(path, "overlayfs:", 10))
+	if (!strncmp(path, "overlay:", 8))
 		return true;
 
-	if (!strncmp(path, "overlay:", 8))
+	if (!strncmp(path, "overlayfs:", 10))
 		return true;
 
 	return false;
@@ -521,18 +512,19 @@ int ovl_mount(struct lxc_storage *bdev)
 		ERROR("Failed to allocate memory");
 		return -1;
 	}
+	upper = dup;
+	lower = dup;
+
+	if (strncmp(dup, "overlay:", 8) == 0)
+		lower += 8;
+	else if (strncmp(dup, "overlayfs:", 10) == 0)
+		lower += 10;
+	if (upper != lower)
+		upper = lower;
 
 	/* support multiple lower layers */
-	lower = strstr(dup, ":/");
-	if (!lower) {
-		ERROR("Failed to detect \":/\" in string \"%s\"", dup);
-		free(dup);
-		return -22;
-	}
-
-	lower++;
-	upper = lower;
-	while ((tmp = strstr(++upper, ":/"))) {
+	while ((tmp = strstr(upper, ":/"))) {
+		tmp++;
 		upper = tmp;
 	}
 
@@ -545,7 +537,7 @@ int ovl_mount(struct lxc_storage *bdev)
 	upper++;
 
 	/* if delta doesn't yet exist, create it */
-	ret = mkdir_p(upper, 0755) < 0;
+	ret = mkdir_p(upper, 0755);
 	if (ret < 0 && errno != EEXIST) {
 		SYSERROR("Failed to create directory \"%s\"", upper);
 		free(dup);
@@ -575,9 +567,9 @@ int ovl_mount(struct lxc_storage *bdev)
 		return -22;
 	}
 
-	strncpy(work, upper, lastslashidx + 1);
-	strncpy(work + lastslashidx, "olwork", sizeof("olwork") - 1);
-	work[lastslashidx + sizeof("olwork")] = '\0';
+	memcpy(work, upper, lastslashidx + 1);
+	memcpy(work + lastslashidx, "olwork", sizeof("olwork") - 1);
+	work[lastslashidx + sizeof("olwork") - 1] = '\0';
 
 	ret = parse_mntopts(bdev->mntopts, &mntflags, &mntdata);
 	if (ret < 0) {
@@ -688,14 +680,14 @@ int ovl_umount(struct lxc_storage *bdev)
 	return ret;
 }
 
-char *ovl_get_lower(const char *rootfs_path)
+const char *ovl_get_lower(const char *rootfs_path)
 {
-	char *s1;
+	const char *s1 = rootfs_path;
 
-	s1 = strstr(rootfs_path, ":/");
-	if (!s1)
-		return NULL;
-	s1++;
+	if (strncmp(rootfs_path, "overlay:", 8) == 0)
+		s1 += 8;
+	else if (strncmp(rootfs_path, "overlayfs:", 10) == 0)
+		s1 += 10;
 
 	s1 = strstr(s1, ":/");
 	if (!s1)
@@ -719,22 +711,20 @@ char *ovl_get_rootfs(const char *rootfs_path, size_t *rootfslen)
 	if (!s1)
 		return NULL;
 
-	s2 = strstr(s1, ":/");
-	if (s2) {
-		s2 = s2 + 1;
-		if ((s3 = strstr(s2, ":/")))
-			*s3 = '\0';
-		rootfsdir = strdup(s2);
-		if (!rootfsdir) {
-			free(s1);
-			return NULL;
-		}
-	}
+	s2 = s1;
+	if (strncmp(rootfs_path, "overlay:", 8) == 0)
+		s2 += 8;
+	else if (strncmp(rootfs_path, "overlayfs:", 10) == 0)
+		s2 += 10;
 
+	s3 = strstr(s2, ":/");
+	if (s3)
+		*s3 = '\0';
+
+	rootfsdir = strdup(s2);
+	free(s1);
 	if (!rootfsdir)
-		rootfsdir = s1;
-	else
-		free(s1);
+		return NULL;
 
 	*rootfslen = strlen(rootfsdir);
 
@@ -747,8 +737,9 @@ int ovl_mkdir(const struct mntent *mntent, const struct lxc_rootfs *rootfs,
 	char lxcpath[MAXPATHLEN];
 	char **opts;
 	int ret;
-	size_t arrlen, dirlen, i, len, rootfslen;
+	size_t arrlen, i, len, rootfslen;
 	int fret = -1;
+	size_t dirlen = 0;
 	char *rootfs_dir = NULL, *rootfs_path = NULL, *upperdir = NULL,
 	     *workdir = NULL;
 
@@ -772,8 +763,7 @@ int ovl_mkdir(const struct mntent *mntent, const struct lxc_rootfs *rootfs,
 	}
 
 	if (rootfs_path) {
-		ret =
-		    snprintf(lxcpath, MAXPATHLEN, "%s/%s", lxc_path, lxc_name);
+		ret = snprintf(lxcpath, MAXPATHLEN, "%s/%s", lxc_path, lxc_name);
 		if (ret < 0 || ret >= MAXPATHLEN)
 			goto err;
 
@@ -876,9 +866,6 @@ int ovl_update_abs_paths(struct lxc_conf *lxc_conf, const char *lxc_path,
 
 		if (strstr(mnt_entry, "overlay"))
 			tmp = "upperdir";
-		else if (strstr(mnt_entry, "aufs"))
-			tmp = "br";
-
 		if (!tmp)
 			continue;
 
@@ -970,9 +957,9 @@ static int ovl_do_rsync(const char *src, const char *dest,
 
 	rdata.src = (char *)src;
 	rdata.dest = (char *)dest;
-	if (am_unpriv())
-		ret = userns_exec_1(conf, lxc_rsync_exec_wrapper, &rdata,
-				    "lxc_rsync_exec_wrapper");
+	if (am_guest_unpriv())
+		ret = userns_exec_full(conf, lxc_rsync_exec_wrapper, &rdata,
+				       "lxc_rsync_exec_wrapper");
 	else
 		ret = run_command(cmd_output, sizeof(cmd_output),
 				  lxc_rsync_exec_wrapper, (void *)&rdata);
