@@ -32,7 +32,7 @@ enum {
 	LXC_SYNC_CGROUP,
 	LXC_SYNC_CGROUP_UNSHARE,
 	LXC_SYNC_CGROUP_LIMITS,
-	LXC_SYNC_POST_CGROUP,
+	LXC_SYNC_READY_START,
 	LXC_SYNC_RESTART,
 	LXC_SYNC_POST_RESTART,
 	LXC_SYNC_ERROR = -1 /* Used to report errors from another process */
