@@ -22,18 +22,14 @@
 #include <libgen.h>
 #include <stdint.h>
 #include <stdio.h>
+#include <string.h>
 #include <unistd.h>
-#include <lxc/lxccontainer.h>
 #include <sys/types.h>
 
-#include "arguments.h"
-#include "log.h"
-#include "lxc.h"
-#include "storage.h"
-#include "storage_utils.h"
-#include "utils.h"
+#include <lxc/lxccontainer.h>
 
-lxc_log_define(lxc_create_ui, lxc);
+#include "arguments.h"
+#include "tool_utils.h"
 
 static uint64_t get_fssize(char *s)
 {
@@ -124,7 +120,7 @@ static void create_helpfn(const struct lxc_arguments *args)
 	argv[2] = NULL;
 
 	execv(path, argv);
-	ERROR("Error executing %s -h", path);
+	fprintf(stderr, "Error executing %s -h\n", path);
 	exit(EXIT_FAILURE);
 }
 
@@ -163,7 +159,7 @@ Options :\n\
 \n\
   BDEV options for LVM or Loop (with -B/--bdev lvm/loop) :\n\
       --fstype=TYPE             Create fstype TYPE\n\
-                                (Default: ext3)\n\
+                                (Default: ext4)\n\
       --fssize=SIZE[U]          Create filesystem of\n\
                                 size SIZE * unit U (bBkKmMgGtT)\n\
                                 (Default: 1G, default unit: M)\n",
@@ -205,6 +201,22 @@ static bool validate_bdev_args(struct lxc_arguments *a)
 	return true;
 }
 
+static bool is_valid_storage_type(const char *type)
+{
+	if (strcmp(type, "dir") == 0 ||
+	    strcmp(type, "btrfs") == 0 ||
+	    strcmp(type, "loop") == 0 ||
+	    strcmp(type, "lvm") == 0 ||
+	    strcmp(type, "nbd") == 0 ||
+	    strcmp(type, "overlay") == 0 ||
+	    strcmp(type, "overlayfs") == 0 ||
+	    strcmp(type, "rbd") == 0 ||
+	    strcmp(type, "zfs") == 0)
+		return true;
+
+	return false;
+}
+
 int main(int argc, char *argv[])
 {
 	struct lxc_container *c;
@@ -227,7 +239,6 @@ int main(int argc, char *argv[])
 
 	if (lxc_log_init(&log))
 		exit(EXIT_FAILURE);
-	lxc_log_options_no_override();
 
 	if (!my_args.template) {
 		fprintf(stderr, "A template must be specified.\n");
@@ -285,7 +296,7 @@ int main(int argc, char *argv[])
 	if (my_args.configfile)
 		c->load_config(c, my_args.configfile);
 	else
-		c->load_config(c, lxc_global_config_value("lxc.default_config"));
+		c->load_config(c, lxc_get_global_config_item("lxc.default_config"));
 
 	if (my_args.fstype)
 		spec.fstype = my_args.fstype;
@@ -323,12 +334,11 @@ int main(int argc, char *argv[])
 		flags = LXC_CREATE_QUIET;
 
 	if (!c->create(c, my_args.template, my_args.bdevtype, &spec, flags, &argv[optind])) {
-		ERROR("Error creating container %s", c->name);
+		fprintf(stderr, "Error creating container %s\n", c->name);
 		lxc_container_put(c);
 		exit(EXIT_FAILURE);
 	}
 
 	lxc_container_put(c);
-	INFO("container %s created", c->name);
 	exit(EXIT_SUCCESS);
 }
