@@ -145,8 +145,8 @@ static int test_console(const char *lxcpath,
 		goto out2;
 	}
 	c->load_config(c, NULL);
-	c->set_config_item(c, "lxc.tty", TTYCNT_STR);
-	c->set_config_item(c, "lxc.pts", "1024");
+	c->set_config_item(c, "lxc.tty.max", TTYCNT_STR);
+	c->set_config_item(c, "lxc.pty.max", "1024");
 	c->save_config(c, NULL);
 	c->want_daemonize(c, true);
 	if (!c->startl(c, 0, NULL)) {
