@@ -27,16 +27,14 @@
 #include <fcntl.h>
 #include <inttypes.h>
 #include <limits.h>
-#include <pthread.h>
 #include <sched.h>
-#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <sys/mount.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <unistd.h>
 
 #include "lxctest.h"
 #include "utils.h"
@@ -149,13 +147,13 @@ void test_detect_ramfs_rootfs(void)
 		goto non_test_error;
 	}
 
-	fd1 = lxc_make_tmpfile(tmpf1, false);
+	fd1 = mkstemp(tmpf1);
 	if (fd1 < 0) {
 		lxc_error("%s\n", "Could not create temporary file.");
 		goto non_test_error;
 	}
 
-	fd2 = lxc_make_tmpfile(tmpf2, false);
+	fd2 = mkstemp(tmpf2);
 	if (fd2 < 0) {
 		lxc_error("%s\n", "Could not create temporary file.");
 		goto non_test_error;
@@ -182,6 +180,7 @@ void test_detect_ramfs_rootfs(void)
 	}
 	fclose(fp1);
 	fp1 = NULL;
+	fd1 = -1;
 
 	/* Test if it correctly fails to detect when no - rootfs rootfs */
 	for (i = 0; i < sizeof(mountinfo) / sizeof(mountinfo[0]); i++) {
@@ -194,6 +193,7 @@ void test_detect_ramfs_rootfs(void)
 	}
 	fclose(fp2);
 	fp2 = NULL;
+	fd2 = -1;
 
 	if (mount(tmpf1, "/proc/self/mountinfo", NULL, MS_BIND, 0) < 0) {
 		lxc_error("%s\n", "Could not overmount \"/proc/self/mountinfo\".");
@@ -443,12 +443,6 @@ void test_parse_byte_size_string(void)
 		exit(EXIT_FAILURE);
 	}
 
-	ret = parse_byte_size_string("1GB", &n);
-	if (ret < 0)
-		exit(EXIT_FAILURE);
-	if (n != 1073741824)
-		exit(EXIT_FAILURE);
-
 	ret = parse_byte_size_string("1TB", &n);
 	if (ret == 0) {
 		lxc_error("%s\n", "Failed to parse \"1TB\"");
@@ -485,12 +479,6 @@ void test_parse_byte_size_string(void)
 		exit(EXIT_FAILURE);
 	}
 
-	ret = parse_byte_size_string("1 GB", &n);
-	if (ret < 0)
-		exit(EXIT_FAILURE);
-	if (n != 1073741824)
-		exit(EXIT_FAILURE);
-
 	ret = parse_byte_size_string("1 TB", &n);
 	if (ret == 0) {
 		lxc_error("%s\n", "Failed to parse \"1 TB\"");
@@ -504,65 +492,19 @@ void test_parse_byte_size_string(void)
 	}
 }
 
-void test_task_blocks_signal(void)
+void test_lxc_config_net_hwaddr(void)
 {
-	int ret;
-	pid_t pid;
+	bool lxc_config_net_hwaddr(const char *line);
 
-	pid = fork();
-	if (pid < 0)
-		_exit(EXIT_FAILURE);
+	if (!lxc_config_net_hwaddr("lxc.net.0.hwaddr = 00:16:3e:04:65:b8\n"))
+		exit(EXIT_FAILURE);
 
-	if (pid == 0) {
-		int i;
-		sigset_t mask;
-		int signals[] = {SIGBUS,   SIGILL,       SIGSEGV,
-				 SIGWINCH, SIGQUIT,      SIGUSR1,
-				 SIGUSR2,  SIGRTMIN + 3, SIGRTMIN + 4};
-
-		sigemptyset(&mask);
-
-		for (i = 0; i < (sizeof(signals) / sizeof(signals[0])); i++) {
-			ret = sigaddset(&mask, signals[i]);
-			if (ret < 0)
-				_exit(EXIT_FAILURE);
-		}
-
-		ret = pthread_sigmask(SIG_BLOCK, &mask, NULL);
-		if (ret < 0) {
-			lxc_error("%s\n", "Failed to block signals");
-			_exit(EXIT_FAILURE);
-		}
-
-		for (i = 0; i < (sizeof(signals) / sizeof(signals[0])); i++) {
-			if (!task_blocks_signal(getpid(), signals[i])) {
-				lxc_error("Failed to detect blocked signal "
-					  "(idx = %d, signal number = %d)\n",
-					  i, signals[i]);
-				_exit(EXIT_FAILURE);
-			}
-		}
-
-		if (task_blocks_signal(getpid(), SIGKILL)) {
-			lxc_error("%s\n",
-				  "Falsely detected SIGKILL as blocked signal");
-			_exit(EXIT_FAILURE);
-		}
-
-		if (task_blocks_signal(getpid(), SIGSTOP)) {
-			lxc_error("%s\n",
-				  "Falsely detected SIGSTOP as blocked signal");
-			_exit(EXIT_FAILURE);
-		}
-
-		_exit(EXIT_SUCCESS);
-	}
-
-	ret = wait_for_pid(pid);
-	if (ret < 0)
-		_exit(EXIT_FAILURE);
-
-	return;
+	if (lxc_config_net_hwaddr("lxc.net"))
+		exit(EXIT_FAILURE);
+	if (lxc_config_net_hwaddr("lxc.net."))
+		exit(EXIT_FAILURE);
+	if (lxc_config_net_hwaddr("lxc.net.0."))
+		exit(EXIT_FAILURE);
 }
 
 int main(int argc, char *argv[])
@@ -575,7 +517,7 @@ int main(int argc, char *argv[])
 	test_lxc_safe_int();
 	test_lxc_safe_long();
 	test_parse_byte_size_string();
-	test_task_blocks_signal();
+	test_lxc_config_net_hwaddr();
 
 	exit(EXIT_SUCCESS);
 }
